@@ -1,67 +1,85 @@
-#' Assess & Save MD-PMS Analyses Definitions
+#' Assess Analyses Definitions
 #'
-#' Define analyses based on an MD-PMS device-event data frame and (optionally)
-#' an MD-PMS exposure data frame.
+#' Define analyses based on an MD-PMS device-event data frame and, optionally,
+#' an MD-PMS exposure data frame. See Details for how to use.
+#'
+#' @param deviceevents A device-events object of class \code{mds_de}, created by
+#' a call to \code{deviceevent()}.
 #'
-#' @param deviceevents A device-events object of class \code{mds_de}
 #' @param device_level String value indicating the source device variable name
-#' to analyze by.
+#' to analyze by. If \code{exposure} is specified, \code{exposure} data will be
+#' matched by \code{device_level}. If a hierarchy of 2 or more are present,
+#' see Details for important information.
 #'
 #' Example: If the \code{deviceevents} variable column is \code{device_1} where
 #' the source variable name for \code{device_1} is \code{'Device Code'}, specify
 #' \code{device_level='Device Code'}.
+#'
 #' @param event_level String value indicating the source event variable name to
 #' analyze by. Note that \code{event_level} is not matched to \code{exposure}.
-#'
-#' Default: \code{NULL} will not analyze by event.
+#' If a hierarchy of 2 or more are present, see Details for important 
+#' information.
 #'
 #' Example: If the \code{deviceevents} variable column is \code{event_1} where
 #' the source variable name for \code{event_1} is \code{'Event Code'}, specify
 #' \code{event_level='Event Code'}.
 #'
+#' Default: \code{NULL} will not analyze by event.
+#'
 #' @param exposure Optional exposure object of class \code{mds_e}. See details
 #' for how exposure analyses definitions are handled.
 #'
 #' Default: \code{NULL} will not consider inclusion of exposure.
+#'
 #' @param date_level String value for the primary date unit to analyze by. Can
 #' be either \code{'months'} or \code{'days'}.
 #'
 #' Default: \code{'months'}
+#'
 #' @param date_level_n Numeric value indicating the number of \code{date_level}s
 #' to analyze by.
 #'
-#' Default: \code{1}
-#'
 #' Example: \code{date_level='months'} and \code{date_level_n=3} indicates
 #' analysis on a quarterly level.
+#'
+#' Default: \code{1}
+#'
 #' @param covariates Character vector specifying names of covariates to also
-#' define analyses for. Acceptable names are covariate variable names from
-#' \code{deviceevents}. Analyses will be defined for each unique level of each
-#' covariate. \code{"_none_"} specifies no covariates, while \code{"_all_"}
-#' specifies all covariates from \code{deviceevents}.
+#' define analyses for. Acceptable names are covariate variables specified
+#' in \code{deviceevents}. If the covariate is a factor, additional subgroup
+#' analyses will be defined at each level of the factor. \code{"_none_"}
+#' specifies no covariates, while \code{"_all_"} are all covariates specified in
+#' \code{deviceevents}. See details for more.
+#'
+#' Example: \code{c("Country", "Region")}
 #'
 #' Default: \code{"_none_"} specifies no covariates.
 #'
-#' Example: \code{"Country"}
 #' @param times_to_calc Integer value indicating the number of date units
 #' counting backwards from the latest date to define analyses for. If
 #' \code{prior} is specified, \code{times_to_calc} will be ignored.
 #'
-#' Default: \code{NULL} will define analyses across all available time.
-#'
 #' Example 1: \code{times_to_calc=12} with \code{date_level="months"} and
 #' \code{date_level_n=1} defines analyses for the last year by month.
 #'
 #' Example 2: \code{times_to_calc=8} with \code{date_level="months"} and
 #' \code{date_level_n=3} defines analyses for the 2 years by quarter.
 #'
+#' Default: \code{NULL} will define analyses across all available time.
+#'
+#' @param invivo Logical value indicating whether to include \code{time_invivo}
+#' from \code{deviceevents} in the analysis definition. See details for more.
+#'
+#' Default: \code{FALSE} will not include \code{time_invivo} in the analysis
+#' definition.
+#'
 #' @param prior Future placeholder, currently not used.
 #'
 #' @return A list of defined analyses of class \code{mds_das}.
 #' Each list item, indexed by a numeric key, defines a set of analyses for a
 #' unique combination of device, event, and covariate level. Each list item is
 #' of the class \code{mds_da}.
-#' Attributes are as follows:
+#' Attributes of class \code{mds_das} are as follows:
 #' \describe{
 #'   \item{date_level}{Defined value for \code{date_level}}
 #'   \item{date_level_n}{Defined value for \code{date_level_n}}
@@ -71,14 +89,59 @@
 #'   \item{prior_used}{Boolean for whether \code{prior} was specified.}
 #'   \item{timestamp}{System time when the analyses were defined.}
 #' }
-#' @details The analyses definitions will always include rollup levels for each
+#'
+#' @details \code{define_analyses()} is a prerequisite to calling
+#' \code{time_series()}. This function enumerates all possible analyses based
+#' on input device-event (\code{deviceevent()}) and, optionally, 
+#' exposure (\code{exposure()}) data frames. An analysis is defined as a set of
+#' instructions specifying at minimum the device level, event level, the date
+#' range of analysis, and the date unit. Additional instructions include the
+#' covariate level, time in-vivo status, and exposure levels.
+#' 
+#' By separating the analysis enumeration (\code{define_analyses()}) from the
+#' generation of the time series (\code{time_series()}), the user may rerun
+#' the analyses on different datasets and/or filter the analyses to only those
+#' of interest.
+#' 
+#' The analyses definitions will always include rollup levels for each
 #' of \code{device_level}, \code{event_level} (if specified), and
-#' \code{covariates}. These rollup analyses will be indicated by the keyword
-#' 'All', while the rollup of all covariates will be called 'Data'.
+#' \code{covariates}. Rollups are analyses at all device, event, and/or
+#' covariate levels. These rollup analyses will be indicated by the keyword
+#' 'All' in the analysis definition.
+#' 
+#' When a hierarchy of 2 or more variables for either \code{device_level} or
+#' \code{event_level} are present in \code{deviceevents}, 
+#' \code{define_analyses()} will enforce the 1-level-up parent level ONLY.
+#' Additional higher parent levels are not currently enforced, thus the user is 
+#' advised to uniquely name the 1-level-up parent level. The parent level
+#' DOES NOT ROLLUP currently because the parent level is intended to separate
+#' disparate data and devices. This may change in the future.
 #'
 #' If \code{exposure} is specified, any available \code{match_levels} will be
 #' used to calculate the appropriate timeframe for analyses. The exception are
 #' the special rollup analyses (see prior paragraph).
+#'
+#' When \code{covariates} are specified, a special rollup analysis definition
+#' will always be defined that does not consider the covariates at all. This
+#' analysis can be identified by \code{covariate='Data'} and
+#' \code{covariate_level='All'} in the output \code{mds_da} object.
+#'
+#' When \code{covariates} are specified and there is no variation in the
+#' distribution of covariate values (e.g. all males, all 10, all missing) in the
+#' device- and event-specific dataset, these specific analyses will be dropped.
+#'
+#' When factor \code{covariates} are specified, covariate-level analyses may be
+#' defined two ways: 1) detect an overall covariate level effect,
+#' also known as a 3-dimensional analysis, and 2) subset the data by each
+#' level of the covariate, also known as a subgroup analysis. 1) will be
+#' denoted as \code{covariate_level='All'} in the output \code{mds_da} object,
+#' while 2) will specify the factor level in \code{covariate_level}.
+#'
+#' If \code{invivo=TRUE}, \code{define_analyses()} will first verify if data
+#' exists in the \code{time_invivo} variable for the given \code{device_level},
+#' \code{event_level}, and, if applicable, \code{covariates} level. If no data
+#' exists, \code{invivo} will be implicitly assigned to \code{FALSE}.
+#'
 #' @examples
 #' # Device-Events
 #' de <- deviceevent(
@@ -113,6 +176,7 @@ define_analyses <- function(
   date_level_n=1,
   covariates="_none_",
   times_to_calc=NULL,
+  invivo=FALSE,
   prior=NULL
 ){
   # Current possibles
@@ -141,6 +205,7 @@ define_analyses <- function(
                       check_names=char_to_df(
                         names(attributes(deviceevents)$covariates)))
   input_param_checker(times_to_calc, check_class="numeric", max_length=1)
+  input_param_checker(invivo, check_class="logical", max_length=1)
   input_param_checker(prior, check_class=c("mds_da", "mds_das"))
   if (!is.null(times_to_calc)){
     if (times_to_calc < 1 | (times_to_calc %% 1 != 0)){
@@ -251,130 +316,186 @@ define_analyses <- function(
           if (j1 != "<>"){
             devDEev1up <- devDEev[devDEev[[ev_1up]] == j1, ]
           } else devDEev1up <- devDEev
-          # Covariates - Enumerate (calculate the rollup level for the last loop)
-          # ---------------------------------------------------------------------
-          # Enumerate each level of each covariate
+          # Covariates - Enumerate
+          # ----------------------
           if (is.null(covariates)){
             uniq_covs <- list("Data"="All")
           } else{
             uniq_covs <- lapply(covariates, function(x){
-              this <- unique(as.character(devDEev1up[[x]]))
-              this <- this[!is.na(this)]
+              if (is.factor(devDEev1up[[x]])){
+                this <- c(unique(as.character(devDEev1up[[x]])), "All")
+                # Identify no variance cases
+                if (length(this) == 2) this <- "_novar_"
+              } else if (is.numeric(devDEev1up[[x]])){
+                this <- "All"
+                # Identify no variance cases
+                if (length(unique(devDEev1up[[x]])) == 1) this <- "_novar_"
+              }
+              # WARNING! If ever the upstream restriction of no NA's in the
+              # covariates is removed, this will produce NA's as a unique 
+              # level. Subsequent handling of NAs is present but untested.
+              this
             })
             names(uniq_covs) <- covariates
             uniq_covs$Data <- "All" # Set rollup level
           }
-
-          # Save analysis instructions for each level of device, event, covariate
-          # ---------------------------------------------------------------------
+          
+          # Save analysis instructions by each level of device, event, covariate
+          # --------------------------------------------------------------------
           for (k in names(uniq_covs)){
             for (l in uniq_covs[[k]]){
-              # Filter for the current covariate level
-              if (paste(k, l) != "Data All"){
-                devCO <- devDEev1up[devDEev1up[[k]] == l, ]
-              } else devCO <- devDEev1up
-
-              # Non-Exposure Case
-              # -----------------
-              # Establish date range
-              dt_range <- convert_date(range(devCO$time, na.rm=T),
-                                       date_level, date_level_n)
-              names(dt_range) <- c("start", "end")
-              # Build list of instructions
-              this <- list(device_level,
-                           stats::setNames(devCO$device[1], dev_lvl),
-                           attributes(deviceevents)$device_hierarchy[[dev_1up]],
-                           stats::setNames(i1, dev_1up),
-                           ifelse(is.null(ev_lvl),
-                                  attributes(deviceevents)$event_hierarchy[[1]],
-                                  event_level),
-                           stats::setNames(devCO$event[1], ifelse(
-                             is.null(ev_lvl),
-                             names(attributes(deviceevents)$event_hierarchy)[1],
-                             ev_lvl)),
-                           ev_1up_lab,
-                           stats::setNames(
-                             ifelse(j1 != "<>", j1, NA),
-                             ifelse(ev_1up == "<>",
-                                    names(attributes(deviceevents)$event_hierarchy)[ev_index],
-                                    ev_1up)),
-                           k, l, dt_range)
-              names(this) <- c("device_level_source", "device_level",
-                               "device_1up_source", "device_1up",
-                               "event_level_source", "event_level",
-                               "event_1up_source", "event_1up",
-                               "covariate", "covariate_level",
-                               "date_range_de")
-
-              # Exposure Case
-              # -------------
-              if (is.null(exposure)){ thes <- data.frame() } else thes <- exposure
-              dev_level_e <- dev_1up_e <- cov_level_e <- stats::setNames(NA, NA)
-              # Filter by current device level
-              if (nrow(thes) > 0 &
-                  this$device_level_source %in%
-                  attributes(exposure)$device_hierarchy){
-                dev_label <- names(which(attributes(exposure)$device_hierarchy ==
-                                           this$device_level_source))
-                dev_level_e <- stats::setNames(as.character(this$device_level),
-                                               dev_label)
-                if (dev_level_e != "All"){
-                  thes <- thes[thes[[names(dev_level_e)]] == dev_level_e, ]
-                  if (nrow(thes) == 0) dev_level_e <- stats::setNames(NA, dev_label)
-                }
+              # If no variance for this covariate, skip
+              skip <- F
+              if (!is.na(l)){
+                if (l == "_novar_") skip <- T
               }
-              this$exp_device_level <- dev_level_e
-              # Filter by 1-up device level
-              if (nrow(thes) > 0 &
-                  this$device_1up_source %in%
-                  attributes(exposure)$device_hierarchy){
-                dev_label <- names(which(attributes(exposure)$device_hierarchy ==
-                                           this$device_1up_source))
-                dev_1up_e <- stats::setNames(as.character(this$device_1up),
-                                               dev_label)
-                if (!is.na(dev_1up_e)){
-                  if (dev_1up_e != "All"){
-                    thes <- thes[thes[[names(dev_1up_e)]] == dev_1up_e, ]
-                    if (nrow(thes) == 0) dev_1up_e <- stats::setNames(NA, dev_label)
+              if (!skip){
+                # Entire analysis requires:
+                # 1. Data All level: covariates not considered (the rollup level as
+                #    the last loop)
+                # Each covariate requires:
+                # 2. Marginal level: analyze for effects of the covariate as a whole
+                # 3. Nominal level (optional): subset by each nominal/binary type
+                #    variable
+                if (is.na(l) & is.factor(devDEev1up[[k]])){ # NA Nominal level
+                  devCO <- devDEev1up[is.na(devDEev1up[[k]]), ]
+                } else if (l %in% c("All")){ # Marginal/Data All level & numeric
+                  devCO <- devDEev1up 
+                } else if (is.factor(devDEev1up[[k]])){ # Nominal level
+                  devCO <- devDEev1up[devDEev1up[[k]] == l, ]
+                } else stop("Unknown covariate filtering specification")
+                
+                # If only 1 row of data remains, skip
+                if (nrow(devCO) > 1){
+                  # Verify time in-vivo variable has variance
+                  vivovar <- F
+                  if (!is.null(attributes(deviceevents)$time_invivo)){
+                    if (invivo & length(unique(devCO$time_invivo)) > 1){
+                      vivovar <- T
+                    }
                   }
+                  
+                  # Assemble output starting with non-exposure data
+                  # -----------------------------------------------
+                  # Establish date range
+                  dt_range <- convert_date(range(devCO$time, na.rm=T),
+                                           date_level, date_level_n)
+                  names(dt_range) <- c("start", "end")
+                  # Build list of instructions
+                  this <- list(
+                    z,
+                    device_level,
+                    stats::setNames(as.character(devCO$device[1]), dev_lvl),
+                    attributes(deviceevents)$device_hierarchy[[dev_1up]],
+                    stats::setNames(i1, dev_1up),
+                    ifelse(is.null(ev_lvl),
+                           attributes(deviceevents)$event_hierarchy[[1]],
+                           event_level),
+                    stats::setNames(devCO$event[1], ifelse(
+                      is.null(ev_lvl),
+                      names(attributes(deviceevents)$event_hierarchy)[1],
+                      ev_lvl)),
+                    ev_1up_lab,
+                    stats::setNames(
+                      ifelse(j1 != "<>", j1, NA),
+                      ifelse(ev_1up == "<>",
+                             names(attributes(
+                               deviceevents)$event_hierarchy)[ev_index],
+                             ev_1up)),
+                    k, l,
+                    vivovar,
+                    attributes(dt_range)$adder,
+                    dt_range)
+                  names(this) <- c("id",
+                                   "device_level_source", "device_level",
+                                   "device_1up_source", "device_1up",
+                                   "event_level_source", "event_level",
+                                   "event_1up_source", "event_1up",
+                                   "covariate", "covariate_level",
+                                   "invivo",
+                                   "date_adder",
+                                   "date_range_de")
+                  
+                  # Exposure Case
+                  # -------------
+                  if (is.null(exposure)){
+                    thes <- data.frame()
+                  } else thes <- exposure
+                  dev_level_e <- dev_1up_e <- cov_level_e <- stats::setNames(NA,
+                                                                             NA)
+                  # Filter by current device level
+                  if (nrow(thes) > 0 &
+                      this$device_level_source %in%
+                      attributes(exposure)$device_hierarchy){
+                    dev_label <- names(which(attributes(
+                      exposure)$device_hierarchy == this$device_level_source))
+                    dev_level_e <- stats::setNames(
+                      as.character(this$device_level), dev_label)
+                    if (dev_level_e != "All"){
+                      thes <- thes[thes[[names(dev_level_e)]] == dev_level_e, ]
+                      if (nrow(thes) == 0){
+                        dev_level_e <- stats::setNames(NA, dev_label)
+                      }
+                    }
+                  }
+                  this$exp_device_level <- dev_level_e
+                  # Filter by 1-up device level
+                  if (nrow(thes) > 0 &
+                      this$device_1up_source %in%
+                      attributes(exposure)$device_hierarchy){
+                    dev_label <- names(which(attributes(
+                      exposure)$device_hierarchy == this$device_1up_source))
+                    dev_1up_e <- stats::setNames(as.character(this$device_1up),
+                                                 dev_label)
+                    if (!is.na(dev_1up_e)){
+                      if (dev_1up_e != "All"){
+                        thes <- thes[thes[[names(dev_1up_e)]] == dev_1up_e, ]
+                        if (nrow(thes) == 0){
+                          dev_1up_e <- stats::setNames(NA, dev_label)
+                        }
+                      }
+                    }
+                  }
+                  this$exp_device_1up <- dev_1up_e
+                  # Filter by event
+                  # NOT IMPLEMENTED <A possible future feature, if requested.>
+                  # Filter for the current covariate level
+                  if (nrow(thes) > 0 &
+                      this$covariate %in% attributes(exposure)$match_levels){
+                    cov_level_e <- stats::setNames(l, k)
+                    if (is.na(l)){ # covariate level is NA case
+                      thes <- thes[is.na(thes[[k]]), ]
+                    } else if (l != "All"){ # covariate level is nominal, non-NA
+                      thes <- thes[as.character(thes[[k]]) == l, ]
+                    } # marginal & Data All levels implied by not filtering
+                    if (nrow(thes) > 0) this$exp_covariate_level <- cov_level_e
+                  }
+                  # Establish exposure date range, if exposure data exists
+                  if (nrow(thes) > 0){
+                    dt_range <- convert_date(range(thes$time, na.rm=T),
+                                             date_level, date_level_n)
+                    names(dt_range) <- c("start", "end")
+                    this$date_range_exposure <- dt_range
+                  } else this$date_range_exposure <- c(as.Date(NA), as.Date(NA))
+                  # Establish date range if exposure is to be used in analysis
+                  # If exposure is not used, date range is the same as
+                  # device-events
+                  dt_range <- c(
+                    max(c(this$date_range_de[1], this$date_range_exposure[1]),
+                        na.rm=T),
+                    min(c(this$date_range_de[2], this$date_range_exposure[2]),
+                        na.rm=T))
+                  dt_range <- convert_date(dt_range, date_level, date_level_n)
+                  names(dt_range) <- c("start", "end")
+                  this$date_range_de_exp <- dt_range
+                  
+                  # Finally, save the analysis
+                  # --------------------------
+                  class(this) <- append(class(this), "mds_da")
+                  out[[z]] <- this
+                  z <- z + 1
                 }
               }
-              this$exp_device_1up <- dev_1up_e
-              # Filter by event
-              # .... <A possible future feature, if requested.>
-              # Filter for the current covariate level
-              if (nrow(thes) > 0 &
-                  this$covariate %in% attributes(exposure)$match_levels){
-                cov_level_e <- stats::setNames(as.character(this$covariate_level),
-                                               this$covariate)
-                if (cov_level_e != "All"){
-                  thes <- thes[thes[[names(cov_level_e)]] == cov_level_e, ]
-                  if (nrow(thes) == 0) cov_level_e <- stats::setNames(
-                    NA, this$covariate)
-                }
-              } else if (paste(k, l) != "Data All") thes <- data.frame()
-              this$exp_covariate_level <- cov_level_e
-              # Establish exposure date range, if exposure data exists
-              if (nrow(thes) > 0){
-                dt_range <- convert_date(range(thes$time, na.rm=T),
-                                         date_level, date_level_n)
-                names(dt_range) <- c("start", "end")
-                this$date_range_exposure <- dt_range
-              } else this$date_range_exposure <- c(as.Date(NA), as.Date(NA))
-              # Establish date range if exposure is to be used in analysis
-              # If exposure is not used, date range is the same as device-events
-              dt_range <- c(
-                max(c(this$date_range_de[1], this$date_range_exposure[1]), na.rm=T),
-                min(c(this$date_range_de[2], this$date_range_exposure[2]), na.rm=T))
-              dt_range <- convert_date(dt_range, date_level, date_level_n)
-              names(dt_range) <- c("start", "end")
-              this$date_range_de_exp <- dt_range
-
-              # Finally, save the analysis
-              # --------------------------
-              class(this) <- append(class(this), "mds_da")
-              out[[z]] <- this
-              z <- z + 1
             }
           }
         }
@@ -416,11 +537,13 @@ define_analyses_dataframe <- function(
   for(j in 1:length(inlist)){
     x <- inlist[[j]]
     for(i in 1:length(x)){
-      date_flag <- F
+      date_flag <- func_flag <- F
       if ("factor" %in% class(x[[i]])){
         x[[i]] <- as.character(x[[i]])
       } else if ("Date" %in% class(x[[i]])){
         date_flag <- T
+      } else if ("function" %in% class(x[[i]])){
+        func_flag <- T
       }
       if (length(x[[i]]) > 1){
         this <- data.frame(t(data.frame(x[i])), stringsAsFactors=F)
@@ -430,26 +553,37 @@ define_analyses_dataframe <- function(
         colnames(this) <- paste0(rownames(this), "_", coln)
       } else if (length(x[[i]]) == 0){
         x[[i]] <- NA
+      } else if (func_flag){
+        x[[i]] <- NA
       } else{
         this <- as.data.frame(x[i], stringsAsFactors=F, row.names="")
       }
       if (date_flag){
         this <- do.call("cbind.data.frame", lapply(this, as.Date))
       }
-      if (exists("out")){
-        out <- cbind.data.frame(out, this)
-      } else{
-        out <- cbind.data.frame(id=j, this)
+      if (all(!is.na(x[[i]]))){
+        if (exists("out")){
+          out <- cbind.data.frame(out, this)
+        } else{
+          # out <- cbind.data.frame(id=j, this)
+          out <- this
+        }
       }
     }
     # If column names are not equal, use the more descriptive set of names
     ncall <- nchar(paste(names(all), collapse=""))
     ncout <- nchar(paste(names(out), collapse=""))
+    
     if (nrow(all) > 0){
       if (ncall > ncout){
-        names(out) <- names(all)
+        # browser()#########################
+
+        out[setdiff(names(all), names(out))] <- NA
+        # names(out) <- names(all)
       } else if (ncout > ncall){
-        names(all) <- names(out)
+        all[setdiff(names(out), names(all))] <- NA
+        
+        # names(all) <- names(out)
       }
     }
     # Combine
