@@ -1,29 +1,41 @@
 context("Define Analyses")
 
+# Set data
+data <- maude
+data$novariance <- c(1, 1, rep(0, nrow(data) - 2))
+invivo <- round(250 * runif(nrow(data)))
+invivo <- ifelse(invivo <= 30, NA, invivo)
+data$invivo <- invivo
+rm(invivo)
+exposures <- sales
+
 # Set params
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name", "device_class"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
   key="report_number",
-  covariates="region",
-  descriptors="_all_")
+  covariates=c("region", "novariance"),
+  descriptors="_all_",
+  time_invivo="invivo")
 Pexp <- exposure(
-  sales,
+  exposures,
   time="sales_month",
   device_hierarchy="device_name",
   match_levels="region",
   count="sales_volume"
 )
 Pdevice_level="device_name"
-Pcovariates="region"
+Pcovariates=c("region", "novariance")
 
 # Reference example
 a1 <- define_analyses(
   Pde, Pdevice_level,
   exposure=Pexp,
-  covariates=Pcovariates)
+  covariates=Pcovariates,
+  invivo=T)
+# temp <- define_analyses_dataframe(a1)
 
 # Basic
 # -----
@@ -93,9 +105,26 @@ test_that("times_to_calc accepts only legal values", {
   expect_is(define_analyses(Pde, Pdevice_level, times_to_calc=2),
             "mds_das")
 })
+test_that("invivo accepts only legal values", {
+  expect_error(define_analyses(Pde, Pdevice_level, invivo="foo"))
+  expect_error(define_analyses(Pde, Pdevice_level, invivo=1.5))
+  expect_error(define_analyses(Pde, Pdevice_level, invivo=as.Date("1971-01-01")))
+  expect_is(define_analyses(Pde, Pdevice_level, invivo=F),
+            "mds_das")
+})
 
 # Attribute check
-test_that("attributes are fully described and consistent", {
+test_that("mds_das attributes are fully described and consistent", {
+  expect_equal(all(names(attributes(a1)) %in% c(
+    "date_level", "date_level_n", "device_level", "prior_used", "timestamp",
+    "class")), T)
+  expect_equal(attributes(a1)$date_level, "months")
+  expect_equal(attributes(a1)$date_level_n, 1)
+  expect_equal(attributes(a1)$device_level, Pdevice_level)
+  expect_equal(attributes(a1)$prior_used, F)
+  expect_is(attributes(a1)$timestamp, "POSIXct")
+})
+test_that("mds_das attributes are fully described and consistent", {
   expect_equal(all(names(attributes(a1)) %in% c(
     "date_level", "date_level_n", "device_level", "prior_used", "timestamp",
     "class")), T)
@@ -112,15 +141,32 @@ test_that("attributes are fully described and consistent", {
 
 test_that("individual analysis is specified as expected", {
   expect_is(a1[[1]], "mds_da")
+  expect_is(a1[[1]]$id, "numeric")
   expect_equal(a1[[1]]$device_level_source, Pdevice_level)
-  expect_equal(a1[[1]]$covariate, Pcovariates)
-  expect_equal(sum(is.na(a1[[1]]$date_range_exposure)), 0)
-  expect_equal(length(a1[[1]]$exp_covariate_level), 1)
-  expect_is(a1[[length(a1)]], "mds_da")
   expect_equal(a1[[length(a1)]]$device_level_source, Pdevice_level)
-  expect_equal(a1[[length(a1)]]$covariate, "Data")
+  expect_is(a1[[1]]$device_level, "character")
+  expect_equal(a1[[1]]$device_1up_source, "device_class")
+  expect_is(a1[[1]]$device_1up, "character")
+  expect_equal(a1[[1]]$event_level_source, "event_type")
+  expect_is(a1[[1]]$event_level, "character")
+  expect_true(is.na(a1[[1]]$event_1up_source))
+  expect_true(is.na(a1[[1]]$event_1up))
+  expect_equal(a1[[1]]$covariate, "region")
+  expect_is(a1[[1]]$covariate_level, "character")
+  expect_is(a1[[1]]$invivo, "logical")
+  expect_is(a1[[1]]$date_adder, "function")
+  expect_is(a1[[1]]$date_range_de, "Date")
+  expect_equal(length(a1[[1]]$date_range_de), 2)
+  expect_is(a1[[1]]$exp_device_level, "character")
+  expect_true(is.na(a1[[1]]$exp_device_1up))
+  expect_null(a1[[length(a1)]]$exp_covariate_level)
+  expect_equal(length(a1[[1]]$exp_covariate_level), 1)
+  expect_equal(sum(is.na(a1[[1]]$date_range_exposure)), 0)
   expect_equal(sum(is.na(a1[[length(a1)]]$date_range_exposure)), 0)
-  expect_equal(length(a1[[length(a1)]]$exp_covariate_level), 1)
+  expect_is(a1[[1]]$date_range_de_exp, "Date")
+  expect_equal(length(a1[[1]]$date_range_de_exp), 2)
+  expect_is(a1[[length(a1)]], "mds_da")
+  expect_equal(a1[[length(a1)]]$covariate, "Data")
 })
 
 
@@ -135,7 +181,7 @@ test_that("barebones individual analysis is specified as expected", {
   expect_equal(a1[[1]]$device_level_source, Pdevice_level)
   expect_equal(a1[[1]]$covariate, "Data")
   expect_equal(a1[[1]]$date_range_exposure, as.Date(c(NA, NA)))
-  expect_equal(a1[[1]]$exp_covariate_level, setNames(NA, NA))
+  expect_null(a1[[1]]$exp_covariate_level)
 })
 
 # Attribute check
@@ -172,7 +218,7 @@ test_that("time change attributes are consistent", {
 
 # Reference example (single level device, no event, covariate)
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
@@ -182,7 +228,8 @@ Pde <- deviceevent(
 Pdevice_level="device_name"
 Pcovariates="region"
 a1 <- define_analyses(
-  Pde, Pdevice_level,
+  Pde, 
+  Pdevice_level,
   exposure=Pexp,
   covariates=Pcovariates)
 test_that("device hierarchy as expected for single-level device", {
@@ -199,7 +246,7 @@ test_that("exposure hierarchy as expected for single-level device", {
 })
 # Variant with last level of hierarchy
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name", "device_class"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
@@ -227,7 +274,7 @@ test_that("exposure hierarchy as expected for single-level device", {
 
 # Multi-level device
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name", "device_class"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
@@ -255,7 +302,7 @@ test_that("exposure hierarchy as expected for multi-level device", {
 
 # Single-level event
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name"),
   event_hierarchy=c("event_type"),
@@ -285,7 +332,7 @@ test_that("exposure hierarchy as expected for single-level event", {
 })
 # Variant with last level of hierarchy
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
@@ -316,7 +363,7 @@ test_that("exposure hierarchy as expected for single-level event", {
 
 # Multi-level event
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
@@ -345,7 +392,7 @@ test_that("exposure hierarchy as expected for multi-level event", {
 
 # Multi-level device, multi-level event
 Pde <- deviceevent(
-  maude,
+  data,
   time="date_received",
   device_hierarchy=c("device_name", "device_class"),
   event_hierarchy=c("event_type", "medical_specialty_description"),
@@ -419,3 +466,4 @@ test_that("Date ranges are populated", {
   expect_is(a2$`Date Ranges`$Start, "Date")
   expect_is(a2$`Date Ranges`$End, "Date")
 })
+
